<?php
namespace Controller;
use Zeedhi\DevOps\Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Request;
use Util\InstanceProvider;

class Label extends \Generated\Controller\Label {

	/**
	 * @var \Util\Utilities $utilities
	 */
	private $utilities;

	/**
     * @var \Service\Task $taskService
     */
    private $taskService;

	/**
	 * @var \Service\Issue $issueService
	 */
	 private $issueService;

    public function __construct(
        Manager $dataSourceManager, \Util\Utilities $utilities, \Service\Task $taskService,
        \Service\Issue $issueService
    ){
		$this->utilities = $utilities;
		parent::__construct($dataSourceManager);
        $this->taskService = $taskService;
		$this->issueService = $issueService;
    }

    /**
     * Rewriting our find() method and setting a condition to FilterCriteria
     *
     * @param DTO\Request\Filter $request   A DTO\Request with a FilterCriteria
     * @param DTO\Response $response		A DTO\Response to be added with DataSets
     */
    public function find(DTO\Request\Filter $request, DTO\Response $response){
        $request->getFilterCriteria()->addCondition("ORGANIZATION", "=", $this->utilities->getCurrentOrganizationId());
        parent::find($request, $response);
    }

    public function findLabelByBoardIssue(DTO\Request\Filter $request, DTO\Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $board = null;
        $projectId = Utilities::getValueInFilterDataSourceByName($param,'PROJECT',true);
        $board = Utilities::getValueInFilterDataSourceByName($param,'BOARD',true);
        $dataset = $this->taskService->findLabelByBoardIssue($projectId,$param, $board);
        $response->addDataSet(new DataSet('Labels', $dataset));
    }

    public function findByBoard(DTO\Request\Filter $request, DTO\Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $projectId = Utilities::getValueInFilterDataSourceByName($param,'PROJECT',true);

        if(empty($projectId)){
            $projectId = 0;
        }
        if(is_array($projectId) && empty($projectId[0])){
            $projectId = 0;
        }
        $dataset = $this->taskService->findLabelsByProject($projectId,$param);
        $response->addDataSet(new DataSet('Labels', $dataset));


    }
    public function findLabelFromLikeProjectName(DTO\Request\Filter $request, DTO\Response $response){
        $filters = $request->getFilterCriteria()->getConditions();

        $dataset = $this->taskService->findLabelFromLikeProjectName($filters);
        $response->addDataSet(new DataSet('Labels', $dataset));
	}

	public function save(DTO\Request $request, DTO\Response $response,$showMessage = true){
        $status = $request->getDataSet()->getRows();


        foreach ($status as &$eachStatus) {
            $eachStatus['ORGANIZATION'] = $this->utilities->getCurrentOrganizationId();
            $eachStatus['CREATED_BY'] = $this->utilities->getCurrentUserId();
            $eachStatus['RESOURCE_T'] =  $eachStatus['IS_PROJECT'] != 'Y' ?  $this->utilities->getCurrentUserId(): null;
            $eachStatus['ID'] = null;
            $eachStatus['PROJECT'] = $eachStatus['IS_PROJECT'] == 'Y' ? (int)$eachStatus['BOARD']: null;
        }
        parent::save(new DTO\Request\DataSet(   new \Zeedhi\Framework\DataSource\DataSet($request->getDataSet()->getDataSourceName(), $status),
												$request->getMethod(), $request->getRoutePath(), $request->getUserId()), $response,false);

        $response->addNotification(new Response\Notification("Rótulo salvo"));
	}

	public function delete(DTO\Request $request, DTO\Response $response,$showMessage = true) {
		parent::delete($request, $response);
        $this->utilities->treatResponseError($response, 'Não foi possível excluir , pois alguma issue
        													está utilizando-o.');
    }

    /**
     * @param \Zeedhi\Framework\DataSource\DataSet $dataSet
     * @return bool
     */
    private function verifyShowMessage(\Zeedhi\Framework\DataSource\DataSet $dataSet) {
        $showMessage = true;

        foreach ($dataSet->getRows() as $row) {
            if(isset($row['isMessage']) && $row['isMessage'] === false) {
                $showMessage = false;
                break;
            }
        }

        return $showMessage;
    }

    public function getLabelsByOperatorId(DTO\Request\Filter $request, DTO\Response $response){
        $filter = $request->getFilterCriteria()->getConditions();
        $labels = $this->issueService->getLabelsByOperatorId($filter);
        $response->addDataSet(new DataSet('labelsByOperator', $labels));
    }

    public function findLabelByBoardIssueFilter(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $data = InstanceProvider::getService()->getLabel()->findLabelByBoardIssueFilter($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('labelByBoardIssueFilter', $data));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

}
