<?php

namespace Factory;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Devops\Entities as DevopsEntities;
use Zeedhi\DevOps\Model\Util\LogicalDeleted;

class Task {

	private $entityManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager){
		$this->entityManager = $entityManager;
	}

    /**
     * @param Entities\Project $project
     * @param $latency
     * @param $name
     * @param \DateTime $estimatedBeginDate
     * @param \DateTime $estimatedEndDate
     * @param $status
     * @param $description
     * @param $percentCompleted
     * @param $duration
     * @param $parent
     * @param $taskStatus
     * @param $dateExceptionType
     * @param $dateException
     * @param string $isIssue
     * @param string $isParent
     * @param null $classification
     * @param string $errorMessage
     * @param null $priority
     * @param null $urlToTest
     * @return Entities\Task
     */
	public function createTask(Entities\Project $project, $latency, $name, \DateTime $estimatedBeginDate,
							   \DateTime $estimatedEndDate, $status, $description, $percentCompleted, $duration, $parent, $taskStatus,
							   $dateExceptionType, $dateException, $isIssue = 'N', $isParent = 'N', $classification = null, $errorMessage = '',
							   $priority = null, $urlToTest = null, Entities\TaskFeatures $taskFeatures = null, $endDate = null,$errorClassification = null, $timePlanned = null, DevopsEntities\ProductVersion $productVersion = null, DevopsEntities\ProductVersion $productVersionDev = null) {
        $entity = new Entities\Task();
		$entity->setName($name);
        $entity->setIsIssue($isIssue);
		$entity->setProject($project);
		$entity->setParent($parent);
		$entity->setEstimatedBeginDate($estimatedBeginDate);
		$entity->setEstimatedEndDate($estimatedEndDate);
		$entity->setStatus($status);
		$entity->setDescription($description);
		$entity->setPercentCompleted($percentCompleted);
		$entity->setDuration($duration);
		$entity->setLatency($latency);
		$entity->setDateException($dateException);
		$entity->setTaskStatus($taskStatus);
		$entity->setDateExceptionType($dateExceptionType);
		$entity->setErrorMessage($errorMessage);
		$entity->setClassification($classification);
		$entity->setIsParent($isParent);
		$entity->setPriority($priority);
		$entity->setUrlToTest($urlToTest);
		$entity->setErrorClassification($errorClassification);
        if ($isIssue == "Y"){
            $entity->setEndDate($endDate);
            $entity->setCurrentTaskStatusIn((new \DateTime()));
        }
		$entity->setTaskFeatures($taskFeatures);
		$entity->setTimePlanned($timePlanned);
		$entity->setProductVersion($productVersion);
		$entity->setProductVersionDev($productVersionDev);
		return $entity;
	}

    /**
     * @param Entities\Task $entity
     * @param Entities\Project $project
     * @param $latency
     * @param $name
     * @param \DateTime $estimatedBeginDate
     * @param \DateTime $estimatedEndDate
     * @param $status
     * @param $description
     * @param $percentCompleted
     * @param $duration
     * @param $parent
     * @param $taskStatus
     * @param $dateExceptionType
     * @param $dateException
     * @param string $isParent
     * @param string $isIssue
     * @param null $classification
     * @param string $errorMessage
     * @param Entities\Priority $priority
     * @param null $urlToTest
     * @param null $response
     * @return Entities\Task
     */
	public function updateTask(Entities\Task $entity, Entities\Project $project, $latency, $name,
                               \DateTime $estimatedBeginDate, \DateTime $estimatedEndDate, $status, $description,
                               $percentCompleted, $duration, $parent, $taskStatus, $dateExceptionType, $dateException,
                               $isParent = 'N', $isIssue = 'N', $classification = null,	$errorMessage = '',
                               Entities\Priority $priority = null, $urlToTest = null, $response = null,
                               $taskFeatures = null, DevopsEntities\ProductVersion $productVersion = null,
                               DevopsEntities\ProductVersion $productVersionDev = null, $issueResponse = null) {

        $entity->setName($name);
        $entity->setIsIssue($isIssue);
		$entity->setProject($project);
		$entity->setParent($parent);
		$entity->setEstimatedBeginDate($estimatedBeginDate);
		$entity->setEstimatedEndDate($estimatedEndDate);
		$entity->setStatus($status);
		$entity->setDescription($description);
		$entity->setPercentCompleted($percentCompleted);
		$entity->setDuration($duration);
		$entity->setLatency($latency);
		$entity->setDateException($dateException);
		$entity->setDateExceptionType($dateExceptionType);
		$entity->setErrorMessage($errorMessage);
		$entity->setClassification($classification);
		$entity->setIsParent($isParent);
		$entity->setPriority($priority);
		$entity->setUrlToTest($urlToTest);
        $entity->setResponse($response);
        $entity->setTaskFeatures($taskFeatures);
        if($isIssue == "Y" && $entity->getTaskStatus() != $taskStatus){
            $entity->setCurrentTaskStatusIn((new \DateTime()));
        }
		$entity->setTaskStatus($taskStatus);
		$entity->setProductVersion($productVersion);
		$entity->setProductVersionDev($productVersionDev);
		$entity->setResponse($issueResponse);
		return $entity;
	}

	/**
	 * @param Entities\Task $entity
	 * @param $percentCompleted
	 * @return Entities\Task
	 */
	public function updatePercentCompleted(Entities\Task $entity,$percentCompleted){

		$entity->setPercentCompleted($percentCompleted);
		return $entity;
	}

	/**
	 * @param Entities\Task $entity
	 * @param \DateTime $date
	 * @return Entities\Task
	 */
	public function insertBeginDate(Entities\Task $entity,\DateTime $date){

		$entity->setBeginDate($date);
		return $entity;
	}

	/**
	 * @param Entities\Task $entity
	 * @param \DateTime $date
	 * @return Entities\Task
	 */
	public function insertEndDate(Entities\Task $entity,\DateTime $date){

		 if($date){
		 	$entity->setEndDate($date);
		 }
		return $entity;
	}

	/**
	 * Set a task as logical deleted. It means that the task is not
	 * really removed from database, but just get his status as
	 * deleted (D)
	 *
	 * @param Entities\Task $entity
	 * @return Entities\Task
	 */
	public function logicalDeleteTask(Entities\Task $entity){
		$entity->setStatus(LogicalDeleted::STATUS_LOGICAL_DELETED);
        $entity->setTaskStatus(null);
		return $entity;
	}

	/**
	 * @param Entities\Task $entity
	 * @param $taskStatus
	 * @return Entities\Task
	 */
	public function updateTaskStatus(Entities\Task $entity, $taskStatus){
		$entity->setTaskStatus($taskStatus);
		return $entity;
	}
	
	
	/**
	 * @param Entities\Task $entity
	 * @param $priority
	 * @return Entities\Task
	 */
	public function updatePriority(Entities\Task $entity,  $priority){
		$entity->setPriority($priority);
		return $entity;
	}

	/**
	 * Sometimes, when you creates a new task, this new
	 * task has a parent task and, so, this parent needs
	 * to be updated too.
	 *
	 * @param $entity
	 * @return Entities\Task
	 */
	public function updateTaskParent(Entities\Task $entity){
		$entity->setIsParent('Y');
		return $entity;
	}

	/**
	 * Since a user can modify his task parameters,
	 * this method updates their changes.
	 *
	 * @param Entities\TaskProperties $entity
	 * @param Entities\TaskStatus $initialTaskStatus
	 * @param Entities\TaskStatus $finalTaskStatus
	 * @return Entities\TaskProperties
	 */
	public function updateTaskOptionsParameter(Entities\TaskProperties $entity,
											   Entities\TaskStatus $initialTaskStatus,
											   Entities\TaskStatus $finalTaskStatus){
		$entity->setInitialTaskStatus($initialTaskStatus);
		$entity->setFinalTaskStatus($finalTaskStatus);
		return $entity;
	}

	/**
	 * This method creates a new parameter when the user
	 * has not one already registered.
	 *
	 * @param Entities\TaskStatus $initialTaskStatus
	 * @param Entities\TaskStatus $finalTaskStatus
	 * @param null $control
	 * @param null $label
	 * @param null $taskType
	 * @param string $isIssue
	 * @return Entities\TaskProperties
	 */
	public function createTaskOptionsParameter(Entities\TaskStatus $initialTaskStatus,
											   Entities\TaskStatus $finalTaskStatus,
											   $control = null, $label = null,
											   $taskType = null, $isIssue = 'N'){
		/** @var Entities\TaskProperties */
		$entity = new Entities\TaskProperties();
		$entity->setControl($control);
		$entity->setLabel($label);
		$entity->setTaskType($taskType);
		$entity->setIsIssue($isIssue);
		$entity->setInitialTaskStatus($initialTaskStatus);
		$entity->setFinalTaskStatus($finalTaskStatus);
		return $entity;
	}

} 